@@ -82,7 +82,12 @@ class sketchpad(base_response_item, keyboard_response_mixin,
 			element_class = getattr(self.element_module(), element_type)
 			element = element_class(self, line)
 			self.elements.append(element)
-		self.elements.sort(key=lambda element: -element.z_index)
+		self.elements.sort(
+			key=lambda element:
+				-element.z_index
+				if isinstance(element.z_index, int)
+				else 0
+		)
 
 	def process_response(self, response_args):
 
@@ -105,17 +110,41 @@ class sketchpad(base_response_item, keyboard_response_mixin,
 			return mouse_response_mixin.prepare_response_func(self)
 		raise osexception(u'Invalid duration: %s' % self.var.duration)
 
+	def _elements(self):
+
+		"""
+		desc:
+			Creates a list of sketchpad elements that are shown, sorted by
+			z-index.
+		"""
+
+		elements = [e for e in self.elements if e.is_shown]
+		try:
+			elements.sort(key=lambda e: -int(self.syntax.eval_text(e.z_index)))
+		except ValueError as e:
+			raise osexception(
+				u'Invalid z_index for sketchpad element',
+				exception=e
+			)
+		return elements
+
 	def prepare(self):
 
 		"""See item."""
 
 		base_response_item.prepare(self)
-		self.canvas = canvas(self.experiment, color=self.var.foreground,
-			background_color=self.var.background, auto_prepare=False)
-		for element in self.elements:
-			if element.is_shown():
-				element.draw()
-		self.canvas.prepare()
+		self.canvas = canvas(
+			self.experiment,
+			color=self.var.foreground,
+			background_color=self.var.background
+		)
+		with self.canvas:
+			for element in self._elements():
+				temp_name = element.draw()
+				if element.element_name is not None:
+					self.canvas.rename_element(
+						temp_name, element.element_name
+					)
 
 	def run(self):
 
@@ -123,7 +152,6 @@ class sketchpad(base_response_item, keyboard_response_mixin,
 
 		self._t0 = self.set_item_onset(self.canvas.show())
 		base_response_item.run(self)
-		del self.canvas
 
 	def coroutine(self):
 
