@@ -19,13 +19,12 @@ along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 
 from libopensesame.py3compat import *
 from pygame.locals import *
-import pygame
 import os
-from libopensesame.exceptions import osexception
-from libopensesame import debug, misc
+import pygame
+import platform
 from openexp.backend import configurable
-from openexp._canvas import canvas
-from openexp._coordinates.legacy import legacy as legacy_coordinates
+from openexp._canvas.canvas import Canvas
+from openexp._coordinates.legacy import Legacy as LegacyCoordinates
 
 # PyGame 1.9.2 suffers from character encoding issues. To get around those, we
 # pass file objects directly to PyGame, instead of paths. These file objects
@@ -40,7 +39,8 @@ from openexp._coordinates.legacy import legacy as legacy_coordinates
 fileobjects = []
 fonts = {}
 
-class legacy(canvas.canvas, legacy_coordinates):
+
+class Legacy(Canvas, LegacyCoordinates):
 
 	"""
 	desc:
@@ -76,31 +76,14 @@ class legacy(canvas.canvas, legacy_coordinates):
 
 	def __init__(self, experiment, auto_prepare=True, **style_args):
 
-		canvas.canvas.__init__(self, experiment, auto_prepare=auto_prepare,
+		Canvas.__init__(self, experiment, auto_prepare=auto_prepare,
 			**style_args)
-		legacy_coordinates.__init__(self)
+		LegacyCoordinates.__init__(self)
 		self.antialias = True
 		self.surface = self.experiment.surface.copy()
 		self.clear()
-
-	def set_config(self, **cfg):
-
-		canvas.canvas.set_config(self, **cfg)
-		for key, val in cfg.items():
-			if u'font_' in key:
-				self._font = None
-		if self._font is None:
-			# First see if the font refers to a file in the resources/ filepool
-			self._font = self._pygame_font(self.experiment, self.font_family,
-				self.font_size)
-			self._font.set_bold(self.font_bold)
-			self._font.set_italic(self.font_italic)
-			self._font.set_underline(self.font_underline)
-
-	def copy(self, canvas):
-
-		self.surface = canvas.surface.copy()
-		self.set_config(**canvas.get_config())
+		if platform.system() == u'Darwin':
+			self.show = self._show_macos
 
 	def show(self):
 
@@ -109,119 +92,64 @@ class legacy(canvas.canvas, legacy_coordinates):
 		pygame.display.flip()
 		return pygame.time.get_ticks()
 
+	def _show_macos(self):
+
+		"""
+		visible: False
+
+		desc:
+			On Mac OS, the display is sometimes not refreshed unless there is
+			some interaction with the event loop. Therefor we implement this
+			hack which is only used on Mac OS.
+		"""
+
+		self.experiment.surface.blit(self.surface, (0, 0))
+		self.experiment.last_shown_canvas = self.surface
+		pygame.display.flip()
+		pygame.event.pump()
+		return pygame.time.get_ticks()
+
+	def prepare(self):
+
+		"""
+		desc:
+			Finishes pending canvas operations (if any), so that a subsequent
+			call to [canvas.show] is extra fast. It's only necessary to call
+			this function if you have disabled `auto_prepare` in
+			[canvas.__init__].
+		"""
+
+		self.surface.fill(self.background_color.backend_color)
+		Canvas.prepare(self)
+
+	def redraw(self):
+
+		if not self.auto_prepare:
+			return
+		self.prepare()
+
+	def set_config(self, **cfg):
+
+		Canvas.set_config(self, **cfg)
+		if hasattr(self, u'surface'):
+			self.redraw()
+
+	def copy(self, canvas):
+
+		self.surface = canvas.surface.copy()
+		Canvas.copy(self, canvas)
+
 	@configurable
 	def clear(self):
 
 		self.surface.fill(self.background_color.backend_color)
+		self._elements = {}
 
-	@configurable
-	def line(self, sx, sy, ex, ey):
-
-		sx, sy = self.to_xy(sx, sy)
-		ex, ey = self.to_xy(ex, ey)
-		pygame.draw.line(self.surface, self.color.backend_color, (sx, sy),
-			(ex, ey), self.penwidth)
-
-	@configurable
-	def rect(self, x, y, w, h):
-
-		x, y = self.to_xy(x, y)
-		if self.fill:
-			pygame.draw.rect(self.surface, self.color.backend_color,
-				(x, y, w, h), 0)
-		else:
-			pygame.draw.rect(self.surface, self.color.backend_color,
-				(x, y, w, h), self.penwidth)
-
-	@configurable
-	def ellipse(self, x, y, w, h):
-
-		x = int(x)
-		y = int(y)
-		w = int(w)
-		h = int(h)
-		x, y = self.to_xy(x, y)
-		if self.fill:
-			pygame.draw.ellipse(self.surface, self.color.backend_color,
-				(x, y, w, h), 0)
-		else:
-			# Because the default way of drawing thick lines gives ugly results
-			# for ellipses, we draw thick ellipses manually, by drawing an
-			# ellipse with the background color inside of it
-			i = self.penwidth / 2
-			j = self.penwidth - i
-			pygame.draw.ellipse(self.surface, self.color.backend_color,
-				(x-i, y-i, w+2*i, h+2*i), 0)
-			pygame.draw.ellipse(self.surface,
-				self.background_color.backend_color, (x+j, y+j, w-2*j, h-2*j),
-				0)
-
-	@configurable
-	def polygon(self, vertices):
-
-		vertices = [self.to_xy(x, y) for x, y in vertices]
-		if self.fill:
-			penwidth = 0
-		else:
-			penwidth = self.penwidth
-		pygame.draw.polygon(self.surface, self.color.backend_color, vertices,
-			penwidth)
-
-	def _text(self, text, x, y):
-
-		surface = self._font.render(text, self.antialias,
-			self.color.backend_color)
-		x, y = self.to_xy(x, y)
-		self.surface.blit(surface, (x, y))
 
 	def _text_size(self, text):
 
 		return self._font.size(text)
 
-	def image(self, fname, center=True, x=None, y=None, scale=None):
-
-		fname = safe_decode(fname)
-		if not os.path.isfile(fname):
-			raise osexception(u'"%s" does not exist' % fname)
-		with open(fname, u'rb') as fd:
-			try:
-				surface = pygame.image.load(fd)
-			except pygame.error:
-				raise osexception(
-					u"'%s' is not a supported image format" % fname)
-		if scale is not None:
-			try:
-				surface = pygame.transform.smoothscale(surface,
-					(int(surface.get_width()*scale),
-					int(surface.get_height()*scale)))
-			except:
-				debug.msg(u"smooth scaling failed for '%s'" % fname,
-					reason=u"warning")
-				surface = pygame.transform.scale(surface,
-					(int(surface.get_width()*scale),
-					int(surface.get_height()*scale)))
-		size = surface.get_size()
-		x, y = self.to_xy(x, y)
-		if center:
-			x -= size[0] / 2
-			y -= size[1] / 2
-		self.surface.blit(surface, (x, y))
-
-	def gabor(self, x, y, orient, freq, env=u"gaussian", size=96, stdev=12,
-		phase=0, col1=u"white", col2=u"black", bgmode=u"avg"):
-
-		surface = canvas._gabor(orient, freq, env, size, stdev, phase, col1,
-			col2, bgmode)
-		x, y = self.to_xy(x, y)
-		self.surface.blit(surface, (x - 0.5 * size, y - 0.5 * size))
-
-	def noise_patch(self, x, y, env=u"gaussian", size=96, stdev=12,
-		col1=u"white", col2=u"black", bgmode=u"avg"):
-
-		surface = canvas._noise_patch(env, size, stdev, col1, col2, bgmode)
-		x, y = self.to_xy(x, y)
-		self.surface.blit(surface, (x - 0.5 * size, y - 0.5 * size))
-
 	@staticmethod
 	def init_display(experiment):
 
@@ -276,8 +204,6 @@ class legacy(canvas.canvas, legacy_coordinates):
 		pygame.display.set_caption(u'OpenSesame (legacy backend)')
 		pygame.mouse.set_visible(False)
 		experiment.surface = pygame.display.get_surface()
-		experiment.font = legacy._pygame_font(experiment, experiment.var.font_family,
-			experiment.var.font_size)
 
 	@staticmethod
 	def close_display(experiment):
@@ -288,35 +214,6 @@ class legacy(canvas.canvas, legacy_coordinates):
 			fonts.pop(list(fonts.keys())[0], None)
 		pygame.display.quit()
 
-	@staticmethod
-	def _pygame_font(experiment, family, size):
 
-		"""
-		visible: False
-
-		desc:
-			A helper function to create a pygame.font.Font object.
-
-		arguments:
-			experiment:	The experiment object.
-			family:		The font family.
-			size:		The font size.
-
-		returns:
-			type:	Font
-		"""
-
-		if (family, size) in fonts:
-			return fonts[(family, size)]
-		try:
-			path = experiment.resource(u'%s.ttf' % family)
-		except:
-			# If the family cannot be found in the filepool, assume that it is
-			# a system font.
-			font = pygame.font.SysFont(family, size)
-		else:
-			fd = open(path, u'rb')
-			fileobjects.append(fd)
-			font = pygame.font.Font(fd, size)
-		fonts[(family, size)] = font
-		return font
+# Non PEP-8 alias for backwards compatibility
+legacy = Legacy
